<?php

namespace Vidgyor\Category\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Vidgyor\Category\Http\Requests\VideoCategoryRequest;
use Vidgyor\Category\Interfaces\VideoCategoryRepositoryInterface;
use Vidgyor\Category\Models\VideoCategory;

/**
 * Resource controller class for video_category.
 */
class VideoCategoryResourceController extends BaseController
{

    /**
     * Initialize video_category resource controller.
     *
     * @param type VideoCategoryRepositoryInterface $video_category
     *
     * @return null
     */
    public function __construct(VideoCategoryRepositoryInterface $video_category)
    {
        parent::__construct();
        $this->repository = $video_category;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Vidgyor\Category\Repositories\Criteria\VideoCategoryResourceCriteria::class);
    }

    /**
     * Display a list of video_category.
     *
     * @return Response
     */
    public function index(VideoCategoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Vidgyor\Category\Repositories\Presenter\VideoCategoryPresenter::class)
                ->$function();
        }

        $video_categories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('category::video_category.names'))
            ->view('category::video_category.index', true)
            ->data(compact('video_categories', 'view'))
            ->output();
    }

    /**
     * Display video_category.
     *
     * @param Request $request
     * @param Model   $video_category
     *
     * @return Response
     */
    public function show(VideoCategoryRequest $request, VideoCategory $video_category)
    {

        if ($video_category->exists) {
            $view = 'category::video_category.show';
        } else {
            $view = 'category::video_category.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('category::video_category.name'))
            ->data(compact('video_category'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new video_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(VideoCategoryRequest $request)
    {

        $video_category = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('category::video_category.name')) 
            ->view('category::video_category.create', true) 
            ->data(compact('video_category'))
            ->output();
    }

    /**
     * Create new video_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(VideoCategoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $video_category                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('category::video_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('category/video_category/' . $video_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/category/video_category'))
                ->redirect();
        }

    }

    /**
     * Show video_category for editing.
     *
     * @param Request $request
     * @param Model   $video_category
     *
     * @return Response
     */
    public function edit(VideoCategoryRequest $request, VideoCategory $video_category)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('category::video_category.name'))
            ->view('category::video_category.edit', true)
            ->data(compact('video_category'))
            ->output();
    }

    /**
     * Update the video_category.
     *
     * @param Request $request
     * @param Model   $video_category
     *
     * @return Response
     */
    public function update(VideoCategoryRequest $request, VideoCategory $video_category)
    {
        try {
            $attributes = $request->all();

            $video_category->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('category::video_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('category/video_category/' . $video_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('category/video_category/' . $video_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the video_category.
     *
     * @param Model   $video_category
     *
     * @return Response
     */
    public function destroy(VideoCategoryRequest $request, VideoCategory $video_category)
    {
        try {

            $video_category->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('category::video_category.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('category/video_category/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('category/video_category/' . $video_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple video_category.
     *
     * @param Model   $video_category
     *
     * @return Response
     */
    public function delete(VideoCategoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('category::video_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('category/video_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/category/video_category'))
                ->redirect();
        }

    }

    /**
     * Restore deleted video_categories.
     *
     * @param Model   $video_category
     *
     * @return Response
     */
    public function restore(VideoCategoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('category::video_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/category/video_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/category/video_category/'))
                ->redirect();
        }

    }

}
